## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = FALSE,
  warning = FALSE
  )

## ----load_packages, message = FALSE, warning = FALSE--------------------------
# load required packages
library(clustSIGNAL)
library(distances)
library(cluster)
library(aricode)
library(dplyr)
library(ggplot2)
library(patchwork)
library(scattermore)

## ----embryo_data_prep---------------------------------------------------------
data(mEmbryo2)
spe <- SpatialExperiment(assays = list(logcounts = me_expr),
                        colData = me_data, spatialCoordsNames = c("X", "Y"))
spe

## ----embryo_data_columns------------------------------------------------------
colnames(colData(spe))

## ----clustSIGNAL_singleRun----------------------------------------------------
set.seed(100)
samples <- "sample_id"
cells <- "uniqueID"
res_emb <- clustSIGNAL(spe, samples, cells, outputs = "a")

## ----embryo_result_list-------------------------------------------------------
names(res_emb)

## ----embryo_clusters_head-----------------------------------------------------
head(res_emb$clusters, n = 3)

## ----embryo_final_spe---------------------------------------------------------
spe <- res_emb$spe_final
spe

## ----colors-------------------------------------------------------------------
colors <- c("#635547", "#8EC792", "#9e6762", "#FACB12", "#3F84AA", "#0F4A9C", 
           "#ff891c", "#EF5A9D", "#C594BF", "#DFCDE4", "#139992", "#65A83E", 
           "#8DB5CE", "#005579", "#C9EBFB", "#B51D8D", "#532C8A", "#8870ad", 
           "#cc7818", "#FBBE92", "#EF4E22", "#f9decf", "#c9a997", "#C72228", 
           "#f79083", "#F397C0", "#DABE99", "#c19f70", "#354E23", "#C3C388",
           "#647a4f", "#CDE088", "#f7f79e", "#F6BFCB", "#7F6874", "#989898", 
           "#1A1A1A", "#FFFFFF", "#e6e6e6", "#77441B", "#F90026", "#A10037", 
           "#DA5921", "#E1C239", "#9DD84A")

## ----embryo_spatialPlots------------------------------------------------------
df_ent <- as.data.frame(colData(spe))

# spatial plot
spt_clust <- df_ent %>%
  ggplot(aes(x = spatialCoords(spe)[, 1],
             y = -spatialCoords(spe)[, 2])) +
  geom_scattermore(pointsize = 3, aes(colour = clustSIGNAL)) +
  scale_color_manual(values = colors) +
  ggtitle("A") +
  labs(x = "x-coordinate", y = "y-coordinate") +
  guides(color = guide_legend(title = "Clusters",
                              override.aes = list(size = 3))) +
  theme_classic() +
  theme(text = element_text(size = 12))

# calculating median entropy of each cluster
celltype_ent <- df_ent %>%
  group_by(as.character(clustSIGNAL)) %>%
  summarise(meanEntropy = median(entropy))
# reordering clusters by their median entropy
# low to high median entropy
cellOrder <- celltype_ent$meanEntropy
names(cellOrder) <- celltype_ent$`as.character(clustSIGNAL)`
cellOrder <- sort(cellOrder)
df_ent$clustSIGNAL <- factor(df_ent$clustSIGNAL, levels = names(cellOrder))
# box plot of cluster entropy
colors_ent <- colors[as.numeric(names(cellOrder))]
box_clust <- df_ent %>%
  ggplot(aes(x = clustSIGNAL, y = entropy, fill = clustSIGNAL)) +
  geom_boxplot() +
  scale_fill_manual(values = colors_ent) +
  ggtitle("B") +
  labs(x = "clustSIGNAL clusters", y = "Entropy") +
  theme_classic() +
  theme(legend.position = "none",
        text = element_text(size = 12),
        axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1))

spt_clust + box_clust + patchwork::plot_layout(guides = "collect", 
                                               widths = c(2, 3))

## ----embryo_clusterMetrics----------------------------------------------------
# average silhouette width
clusts <- as.numeric(as.character(spe$clustSIGNAL))
cXg_mat <- t(as.matrix(logcounts(spe)))
distMat <- distances(cXg_mat)
silCluster <- as.matrix(silhouette(clusts, distMat))
spe$rcSil <- silCluster[, 3]

# ARI and NMI
as.data.frame(colData(spe)) %>%
  summarise(ARI = aricode::ARI(celltype_mapped_refined, clustSIGNAL),
            NMI = aricode::NMI(celltype_mapped_refined, clustSIGNAL),
            ASW = mean(rcSil))

## ----embryo_entropyMetrics----------------------------------------------------
# Data assessment - Overall entropy
as.data.frame(colData(spe)) %>%
  summarise(min_Entropy = min(entropy),
            max_Entropy = max(entropy),
            mean_Entropy = mean(entropy))

## ----embryo_entropyPlots------------------------------------------------------
# Histogram of entropy spread
hst_ent <- as.data.frame(colData(spe)) %>%
  ggplot(aes(entropy)) +
  geom_histogram(binwidth = 0.05) +
  ggtitle("A") +
  labs(x = "Entropy", y = "Number of regions") +
  theme_classic() +
  theme(text = element_text(size = 12))

# Spatial plot showing sample entropy distribution
spt_ent <- as.data.frame(colData(spe)) %>%
  ggplot(aes(x = spatialCoords(spe)[, 1],
             y = -spatialCoords(spe)[, 2])) +
  geom_scattermore(pointsize = 3,
             aes(colour = entropy)) +
  scale_colour_gradient2("Entropy", low = "grey", high = "blue") +
  scale_size_continuous(range = c(0, max(spe$entropy))) +
  ggtitle("B") +
  labs(x = "x-coordinate", y = "y-coordinate") +
  theme_classic() +
  theme(text = element_text(size = 12))

hst_ent + spt_ent

## ----clustSIGNAL_onlyEntropy--------------------------------------------------
data(mEmbryo2)
spe <- SpatialExperiment(assays = list(logcounts = me_expr),
                         colData = me_data, spatialCoordsNames = c("X", "Y"))
set.seed(100)
spe <- scater::runPCA(spe)
spe <- clustSIGNAL::p1_clustering(spe, dimRed = "PCA", batch = FALSE, 
                                  batch_by = "None", clustParams = list(
                                    0, 0, 30, 5, "louvain"))
outReg <- clustSIGNAL::neighbourDetect(spe, samples = "sample_id", NN = 30, 
                                       cells = "uniqueID", sort = TRUE)
spe <- entropyMeasure(spe, cells = "uniqueID", outReg$regXclust, threads = 1)
head(spe$entropy)

## ----hypothal_data_prep-------------------------------------------------------
data(mHypothal)
spe2 <- SpatialExperiment(assays = list(logcounts = mh_expr),
                        colData = mh_data, spatialCoordsNames = c("X", "Y"))
spe2

## ----hypothal_data_columns----------------------------------------------------
colnames(colData(spe2))

## ----clustSIGNAL_multiRun-----------------------------------------------------
set.seed(110)
samples <- "samples"
cells <- "Cell_ID"
res_hyp <- clustSIGNAL(spe2, samples, cells, threads = 4, outputs = "a")

## ----hypothal_final_spe-------------------------------------------------------
spe2 <- res_hyp$spe_final
spe2

## ----hypothal_samples---------------------------------------------------------
samplesList <- levels(spe2[[samples]])
samplesList

## ----hypothal_clusterMetrics--------------------------------------------------
# calculating silhouette width per sample
silWidthRC <- matrix(nrow = 0, ncol = 3)
for (s in samplesList) {
  speX <- spe2[, spe2[[samples]] == s]
  clust_sub <- as.numeric(as.character(speX$clustSIGNAL))
  cXg <- t(as.matrix(logcounts(speX)))
  distMat <- distances(cXg)
  silCluster <- as.matrix(silhouette(clust_sub, distMat))
  silWidthRC <- rbind(silWidthRC, silCluster)
}
spe2$rcSil <- silWidthRC[, 3]

as.data.frame(colData(spe2)) %>%
  group_by(samples) %>%
  summarise(ARI = aricode::ARI(Cell_class, clustSIGNAL),
            NMI = aricode::NMI(Cell_class, clustSIGNAL),
            ASW = mean(rcSil),
            min_Entropy = min(entropy),
            max_Entropy = max(entropy),
            mean_Entropy = mean(entropy))

## ----hypothal_spatialPlots----------------------------------------------------
df_ent <- as.data.frame(colData(spe2))

# spatial plot
spt_clust <- df_ent %>%
    ggplot(aes(x = spatialCoords(spe2)[, 1],
               y = -spatialCoords(spe2)[, 2])) +
    geom_scattermore(pointsize = 3, aes(colour = clustSIGNAL)) +
    scale_color_manual(values = colors) +
    facet_wrap(vars(samples), scales = "free", nrow = 1) +
    labs(x = "x-coordinate", y = "y-coordinate") +
    guides(color = guide_legend(title = "Clusters",
                                override.aes = list(size = 3))) +
    theme_classic() +
    theme(text = element_text(size = 12),
          axis.text.x = element_text(angle = 90, vjust = 0.5))

box_clust <- list()
for (s in samplesList) {
  df_ent_sub <- as.data.frame(colData(spe2)[spe2[[samples]] == s, ])
  # calculating median entropy of each cluster in a sample
  celltype_ent <- df_ent_sub %>%
    group_by(as.character(clustSIGNAL)) %>%
    summarise(meanEntropy = median(entropy))
  # reordering clusters by their median entropy
  # low to high median entropy
  cellOrder <- celltype_ent$meanEntropy
  names(cellOrder) <- celltype_ent$`as.character(clustSIGNAL)`
  cellOrder = sort(cellOrder)
  df_ent_sub$clustSIGNAL <- factor(df_ent_sub$clustSIGNAL, 
                                   levels = names(cellOrder))

  # box plot of cluster entropy
  colors_ent <- colors[as.numeric(names(cellOrder))]
  box_clust[[s]] <- df_ent_sub %>%
    ggplot(aes(x = clustSIGNAL, y = entropy, fill = clustSIGNAL)) +
    geom_boxplot() +
    scale_fill_manual(values = colors_ent) +
    facet_wrap(vars(samples), nrow = 1) +
    labs(x = "clustSIGNAL clusters", y = "Entropy") +
    ylim(0, NA) +
    theme_classic() +
    theme(strip.text = element_blank(),
          legend.position = "none",
          text = element_text(size = 12),
          axis.text.x = element_text(angle = 90, vjust = 0.5))
}

spt_clust / (patchwork::wrap_plots(box_clust[1:3], nrow = 1) +
               plot_layout(axes = "collect")) +
  plot_layout(guides = "collect", heights = c(5, 3)) +
  plot_annotation(
    title = "Spatial (top) and entropy (bottom) distributions of clusters")

## ----hypothal_entropyPlots----------------------------------------------------
# Histogram of entropy spread
hst_ent <- as.data.frame(colData(spe2)) %>%
  ggplot(aes(entropy)) +
  geom_histogram(binwidth = 0.05) +
  facet_wrap(vars(samples), nrow = 1) +
  labs(x = "Entropy", y = "Number of regions") +
  theme_classic() +
  theme(text = element_text(size = 12))

# Spatial plot showing sample entropy distribution
spt_ent <- as.data.frame(colData(spe2)) %>%
  ggplot(aes(x = spatialCoords(spe2)[, 1],
             y = -spatialCoords(spe2)[, 2])) +
  geom_scattermore(pointsize = 3,
             aes(colour = entropy)) +
  scale_colour_gradient2("Entropy", low = "grey", high = "blue") +
  scale_size_continuous(range = c(0, max(spe2$entropy))) +
  facet_wrap(vars(samples), scales = "free", nrow = 1) +
  labs(x = "x-coordinate", y = "y-coordinate") +
  theme_classic() +
  theme(strip.text = element_blank(),
        text = element_text(size = 12),
        axis.text.x = element_text(angle = 90, vjust = 0.5))

hst_ent / spt_ent + plot_layout(heights = c(3,5)) +
    plot_annotation(
      title = "Entropy spread (top) and spatial distribution (bottom)")

## -----------------------------------------------------------------------------
sessionInfo()

